package net.wanji.opt.common;

import com.alibaba.fastjson.JSONObject;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import lombok.extern.slf4j.Slf4j;
import net.wanji.opt.dto.LineCongestion;
import net.wanji.opt.dto.PhaseEmptyResult;
import org.apache.kafka.clients.consumer.ConsumerRecord;
import org.apache.kafka.clients.consumer.ConsumerRecords;
import org.apache.kafka.clients.consumer.KafkaConsumer;
import org.apache.kafka.clients.consumer.OffsetAndTimestamp;
import org.apache.kafka.common.TopicPartition;
import org.apache.kafka.common.serialization.StringDeserializer;

import java.time.Duration;
import java.util.*;

/**
 * 从 Kafka 中按时间范围消费数据
 *
 * @author Kent HAN
 * @date 2024/4/16 8:45
 */
@Slf4j
public class KafkaConsumerUtil {
    private final KafkaConsumer<String, String> consumer;
    private final ObjectMapper objectMapper;

    public KafkaConsumerUtil(String bootstrapServers, String groupId) {
        Properties props = new Properties();
        props.put("bootstrap.servers", bootstrapServers);
        props.put("group.id", groupId);
        props.put("enable.auto.commit", "true");
        props.put("auto.commit.interval.ms", "1000");
        props.put("key.deserializer", StringDeserializer.class.getName());
        props.put("value.deserializer", StringDeserializer.class.getName());
        this.consumer = new KafkaConsumer<>(props);
        this.objectMapper = new ObjectMapper();
    }

    public List<PhaseEmptyResult> consumeEmptyPhaseForTimeRange(
            String topic, int partition, long startTime, long endTime) {

        List<PhaseEmptyResult> results = new ArrayList<>();
        TopicPartition topicPartition = new TopicPartition(topic, partition);
        consumer.assign(Collections.singletonList(topicPartition));

        HashMap<TopicPartition, Long> timestampToSearch = new HashMap<>();
        timestampToSearch.put(topicPartition, startTime);
        OffsetAndTimestamp offsetAndTimestamp = consumer.offsetsForTimes(timestampToSearch).get(topicPartition);
        if (offsetAndTimestamp == null) {
            return results;
        }
        long startOffset = offsetAndTimestamp.offset();

        // 开始消费
        consumer.seek(topicPartition, startOffset);
        int emptyPollCount = 0; // 记录空轮询次数的计数器
        int maxEmptyPolls = 10; // 设置最大空轮询次数
        try {
            boolean keepConsuming = true;
            while (keepConsuming) {
                ConsumerRecords<String, String> records = consumer.poll(Duration.ofMillis(100));

                if (records.isEmpty()) {
                    emptyPollCount++; // 如果没有记录，增加空轮询计数
                    if (emptyPollCount >= maxEmptyPolls) {
                        // 如果达到最大空轮询次数，退出循环
                        break;
                    }
                } else {
                    emptyPollCount = 0; // 如果有记录，重置空轮询计数器
                    for (ConsumerRecord<String, String> record : records) {
                        long recordTime = record.timestamp();
                        if (recordTime >= startTime && recordTime <= endTime) {
                            PhaseEmptyResult phaseEmptyResult =
                                    objectMapper.readValue(record.value(), PhaseEmptyResult.class);
                            results.add(phaseEmptyResult);
                        } else if (recordTime > endTime) {
                            keepConsuming = false;
                            break;
                        }
                    }
                }
            }
        } catch (JsonProcessingException e) {
            throw new RuntimeException(e);
        } finally {
            consumer.close();
        }
        return results;
    }

    public List<LineCongestion> consumeLineMetricForTimeRange(
            String topic, int partition, long startTime, long endTime) {

        List<LineCongestion> results = new ArrayList<>();
        TopicPartition topicPartition = new TopicPartition(topic, partition);
        consumer.assign(Collections.singletonList(topicPartition));

        HashMap<TopicPartition, Long> timestampToSearch = new HashMap<>();
        timestampToSearch.put(topicPartition, startTime);
        OffsetAndTimestamp offsetAndTimestamp = consumer.offsetsForTimes(timestampToSearch).get(topicPartition);
        if (offsetAndTimestamp == null) {
            return results;
        }
        long startOffset = offsetAndTimestamp.offset();

        // 开始消费
        consumer.seek(topicPartition, startOffset);
        int emptyPollCount = 0; // 记录空轮询次数的计数器
        int maxEmptyPolls = 10; // 设置最大空轮询次数
        try {
            boolean keepConsuming = true;
            while (keepConsuming) {
                ConsumerRecords<String, String> records = consumer.poll(Duration.ofMillis(100));

                if (records.isEmpty()) {
                    emptyPollCount++; // 如果没有记录，增加空轮询计数
                    if (emptyPollCount >= maxEmptyPolls) {
                        // 如果达到最大空轮询次数，退出循环
                        break;
                    }
                } else {
                    emptyPollCount = 0; // 如果有记录，重置空轮询计数器
                    for (ConsumerRecord<String, String> record : records) {
                        long recordTime = record.timestamp();
                        if (recordTime >= startTime && recordTime <= endTime) {
                            LineCongestion lineCongestion = JSONObject.parseObject(record.value(), LineCongestion.class);
                            results.add(lineCongestion);
                        } else if (recordTime > endTime) {
                            keepConsuming = false;
                            break;
                        }
                    }
                }
            }
        } catch (Exception e) {
            throw new RuntimeException(e);
        } finally {
            consumer.close();
        }
        log.info("从kafka中获取干线指标数据 topic:{},startTime:{},endTime:{}");
        return results;
    }
}
