package net.wanji.opt.service.es;

import lombok.extern.slf4j.Slf4j;
import net.wanji.common.constant.Constants;
import net.wanji.common.utils.tool.JacksonUtils;
import net.wanji.common.utils.tool.StringUtils;
import net.wanji.opt.dto.CrossLaneSnapshotDataDTO;
import org.elasticsearch.action.search.SearchRequest;
import org.elasticsearch.action.search.SearchResponse;
import org.elasticsearch.client.RequestOptions;
import org.elasticsearch.client.RestHighLevelClient;
import org.elasticsearch.client.core.CountRequest;
import org.elasticsearch.client.core.CountResponse;
import org.elasticsearch.index.query.*;
import org.elasticsearch.search.SearchHit;
import org.elasticsearch.search.SearchHits;
import org.elasticsearch.search.builder.SearchSourceBuilder;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

/**
 * @author duanruiming
 * @date 2023/10/26 14:13
 */
@Service
@Slf4j
public class LaneSnapshotDataQueryService implements LaneSnapshotService {
    @Resource
    RestHighLevelClient client;

    SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");

    @Override
    public List<CrossLaneSnapshotDataDTO> queryByCrossIdAndTimeSpan(
            String crossId, int startTimeStamp, int endTimeStamp, int pageNum, String laneId, boolean overFlow)
            throws Exception {

        List<CrossLaneSnapshotDataDTO> result = new ArrayList<>();

        try {
            SearchRequest searchRequest = new SearchRequest(Constants.LANE_SNAPSHOT_DATA_ES_INDEX);
            SearchSourceBuilder searchSourceBuilder = new SearchSourceBuilder();
            MatchQueryBuilder matchQuery = QueryBuilders.matchQuery("crossId", crossId);
            RangeQueryBuilder rangeQuery = QueryBuilders.rangeQuery("timeStamp")
                    .gte(startTimeStamp)
                    .lte(endTimeStamp);


            BoolQueryBuilder boolQuery = QueryBuilders.boolQuery()
                    .must(matchQuery)
                    .must(rangeQuery);

            // 是否是溢出事件
            MatchQueryBuilder overFlowQuery = null;
            if (overFlow) {
                overFlowQuery = QueryBuilders.matchQuery("overflow", true);
                boolQuery.must(overFlowQuery);
            }


            // 车道号筛选
            String[] split = null;
            if (StringUtils.isNotBlank(laneId)) {
                split = laneId.split(",");
                TermsQueryBuilder termsQuery = QueryBuilders.termsQuery("laneId.keyword", Arrays.asList(split));
                boolQuery.must(termsQuery);
            }

            if (pageNum == -1) {
                searchSourceBuilder
                        .query(boolQuery)
                        .size(50000);
            } else {
                searchSourceBuilder
                        .query(boolQuery)
                        .from(pageNum * 500)
                        .size((pageNum + 1) * 500);
            }
            searchRequest.source(searchSourceBuilder);

            SearchResponse response = client.search(searchRequest, RequestOptions.DEFAULT);
            SearchHits hits = response.getHits();

            for (SearchHit hit : hits) {
                String sourceAsString = hit.getSourceAsString();
                CrossLaneSnapshotDataDTO crossLaneSnapshotDataDTO =
                        JacksonUtils.getInstance().readValue(sourceAsString, CrossLaneSnapshotDataDTO.class);
                result.add(crossLaneSnapshotDataDTO);
            }
        } catch (Exception e) {
            log.error("ES数据获取错误", e);
            throw new Exception(e);
        }
        return result;
    }

    @Override
    public int queryCountsByCrossIdAndTimeSpan(String crossId, int startTimeStamp, int endTimeStamp) throws Exception {
        try {
            CountRequest countRequest = new CountRequest(Constants.LANE_SNAPSHOT_DATA_ES_INDEX);
            SearchSourceBuilder searchSourceBuilder = new SearchSourceBuilder();

            MatchQueryBuilder matchQuery = QueryBuilders.matchQuery("crossId", crossId);
            RangeQueryBuilder rangeQuery = QueryBuilders.rangeQuery("timeStamp")
                    .gte(startTimeStamp)
                    .lte(endTimeStamp);

            BoolQueryBuilder boolQuery = QueryBuilders.boolQuery()
                    .must(matchQuery)
                    .must(rangeQuery);

            searchSourceBuilder.query(boolQuery);
            countRequest.source(searchSourceBuilder);

            CountResponse countResponse = client.count(countRequest, RequestOptions.DEFAULT);
            long count = countResponse.getCount();

            return (int) count;

        } catch (Exception e) {
            log.error("ES数据获取错误", e);
            throw new Exception(e);
        }
    }
}
