package net.wanji.opt.service.impl;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.ObjectUtil;
import com.fasterxml.jackson.databind.ObjectMapper;
import lombok.extern.slf4j.Slf4j;
import net.wanji.common.enums.CrossStatusEnum;
import net.wanji.common.enums.TurnConvertEnum;
import net.wanji.common.utils.tool.CrossUtil;
import net.wanji.common.utils.tool.StringUtils;
import net.wanji.common.utils.tool.TimeArrayUtil;
import net.wanji.databus.bo.CrossIdAndStartEndDateBO;
import net.wanji.databus.bo.CrossIdBO;
import net.wanji.databus.bo.HeatMapBO;
import net.wanji.databus.dao.entity.*;
import net.wanji.databus.dao.mapper.*;
import net.wanji.databus.dto.MetricHistDTO;
import net.wanji.databus.dto.RunningPlanDTO;
import net.wanji.databus.po.BaseCrossInfoPO;
import net.wanji.databus.po.CrossDataHistPO;
import net.wanji.databus.po.CrossTurnInfoPO;
import net.wanji.databus.po.LaneInfoPO;
import net.wanji.databus.vo.RunningEvaluateCrossListVO;
import net.wanji.opt.bo.CrossNameBO;
import net.wanji.opt.bo.MetricsDetailBO;
import net.wanji.opt.common.KafkaConsumerUtil;
import net.wanji.opt.dto.PhaseEmptyResult;
import net.wanji.opt.service.RunningEvaluateService;
import net.wanji.opt.vo.*;
import org.jetbrains.annotations.NotNull;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.text.DecimalFormat;
import java.text.SimpleDateFormat;
import java.time.Instant;
import java.time.LocalDate;
import java.time.ZoneId;
import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;
import java.util.stream.Stream;

/**
 * @author Kent HAN
 * @date 2023/8/21 9:53
 */
@Slf4j
@Service
public class RunningEvaluateServiceImpl implements RunningEvaluateService {

    @Value("${spring.kafka.bootstrap-servers}")
    private String bootstrapServers;

    @Value("${spring.kafka.empty-phase-topic}")
    private String emptyPhaseTopic;

    private final CrossDataRealtimeMapper crossDataRealtimeMapper;
    private final CrossDataHistMapper crossDataHistMapper;
    private final BaseCrossSectionMapper baseCrossSectionMapper;
    private final BaseCrossSchemeMapper baseCrossSchemeMapper;
    private final BaseCrossDirInfoMapper baseCrossDirInfoMapper;
    private final BaseCrossTurnInfoMapper baseCrossTurnInfoMapper;
    private final LaneInfoMapper laneInfoMapper;
    private final BaseCrossInfoMapper baseCrossInfoMapper;
    private final BaseCrossPlanMapper baseCrossPlanMapper;
    private final BaseCrossSchedulesMapper baseCrossSchedulesMapper;
    private final BaseCrossSchedulesPlanMapper baseCrossSchedulesPlanMapper;
    private final CrossDirDataHistMapper crossDirDataHistMapper;
    private final CrossTurnDataHistMapper crossTurnDataHistMapper;
    private final CrossLaneDataHistMapper crossLaneDataHistMapper;

    SimpleDateFormat HOUR_SDF = new SimpleDateFormat("HH:mm");
    SimpleDateFormat DAY_SDF = new SimpleDateFormat("yyyy-MM-dd");

    ObjectMapper objectMapper = new ObjectMapper();

    public RunningEvaluateServiceImpl(CrossDataRealtimeMapper crossDataRealtimeMapper,
                                      CrossDataHistMapper crossDataHistMapper,
                                      @Qualifier("baseCrossSectionMapper") BaseCrossSectionMapper baseCrossSectionMapper,
                                      @Qualifier("baseCrossSchemeMapper") BaseCrossSchemeMapper baseCrossSchemeMapper,
                                      @Qualifier("baseCrossDirInfoMapper") BaseCrossDirInfoMapper baseCrossDirInfoMapper,
                                      @Qualifier("baseCrossTurnInfoMapper") BaseCrossTurnInfoMapper baseCrossTurnInfoMapper,
                                      @Qualifier("laneInfoMapper") LaneInfoMapper laneInfoMapper,
                                      @Qualifier("baseCrossInfoMapper") BaseCrossInfoMapper baseCrossInfoMapper,
                                      @Qualifier("baseCrossPlanMapper") BaseCrossPlanMapper baseCrossPlanMapper,
                                      @Qualifier("baseCrossSchedulesMapper") BaseCrossSchedulesMapper baseCrossSchedulesMapper,
                                      @Qualifier("baseCrossSchedulesPlanMapper") BaseCrossSchedulesPlanMapper baseCrossSchedulesPlanMapper,
                                      CrossDirDataHistMapper crossDirDataHistMapper,
                                      CrossTurnDataHistMapper crossTurnDataHistMapper,
                                      CrossLaneDataHistMapper crossLaneDataHistMapper) {
        this.crossDataRealtimeMapper = crossDataRealtimeMapper;
        this.crossDataHistMapper = crossDataHistMapper;
        this.baseCrossSectionMapper = baseCrossSectionMapper;
        this.baseCrossSchemeMapper = baseCrossSchemeMapper;
        this.baseCrossDirInfoMapper = baseCrossDirInfoMapper;
        this.baseCrossTurnInfoMapper = baseCrossTurnInfoMapper;
        this.laneInfoMapper = laneInfoMapper;
        this.baseCrossInfoMapper = baseCrossInfoMapper;
        this.baseCrossPlanMapper = baseCrossPlanMapper;
        this.baseCrossSchedulesMapper = baseCrossSchedulesMapper;
        this.baseCrossSchedulesPlanMapper = baseCrossSchedulesPlanMapper;
        this.crossDirDataHistMapper = crossDirDataHistMapper;
        this.crossTurnDataHistMapper = crossTurnDataHistMapper;
        this.crossLaneDataHistMapper = crossLaneDataHistMapper;
    }

    @Override
    public List<RunningEvaluateCrossListVO> crossList(CrossNameBO crossNameBO) {
        String crossName = crossNameBO.getCrossName();
        List<RunningEvaluateCrossListVO> voList = crossDataRealtimeMapper.selectByCrossName(crossName);
        for (RunningEvaluateCrossListVO vo : voList) {
            Double sturation = vo.getSturation();
            vo.setServiceLevel(CrossUtil.getServiceLevel(sturation));
            // 路口经纬度
            String crossId = vo.getCrossId();
            BaseCrossInfoPO baseCrossInfoPO = baseCrossInfoMapper.selectById(crossId);
            String location = baseCrossInfoPO.getLocation();
            double[] lonLat = CrossUtil.getLonLat(location);
            vo.setLongitude(lonLat[0]);
            vo.setLatitude(lonLat[1]);
            // 浮点精度
            BigDecimal congestionIndex = vo.getCongestionIndex();
            BigDecimal roundedCongestionIndex = congestionIndex.setScale(2, RoundingMode.HALF_UP);
            vo.setCongestionIndex(roundedCongestionIndex);
        }
        // 按服务水平降序排序
        List<RunningEvaluateCrossListVO> res = voList.stream()
                .sorted(Comparator.comparing(RunningEvaluateCrossListVO::getSturation).reversed())
                .collect(Collectors.toList());

        return res;
    }

    @Override
    public RunningEvaluateCrossEvaluateVO crossEvaluate(CrossIdAndStartEndDateBO bo) {
        RunningEvaluateCrossEvaluateVO vo = new RunningEvaluateCrossEvaluateVO();

        String crossId = bo.getCrossId();
        List<CrossDataHistPO> crossDataHistPOList = buildCrossDataHistPOList(bo, crossId);

        // 服务水平
        double sumA = crossDataHistPOList.stream()
                .filter(po -> isInTimeRange(po.getBatchTime()))
                .mapToDouble(po -> po.getFlow() * po.getSturation())
                .sum();

        int sumB = crossDataHistPOList.stream()
                .filter(po -> isInTimeRange(po.getBatchTime()))
                .mapToInt(CrossDataHistPO::getFlow)
                .sum();

        double saturation = sumA / sumB;
        String serviceLevel = CrossUtil.getServiceLevel(saturation);
        vo.setServiceLevel(serviceLevel);


        List<CrossDataHistPO> congestionEventList = buildCongestionEvents(crossDataHistPOList); // 获取拥堵事件集合
        vo.setCongestionTimes(congestionEventList.size());
        int congestionSum = congestionEventList.stream()
                .filter(Objects::nonNull)
                .map(CrossDataHistPO::getDuration)
                .filter(Objects::nonNull)
                .mapToInt(Integer::intValue)
                .sum();
        vo.setCongestionSum(congestionSum);

        List<CrossDataHistPO> unbalanceEventList = buildUnbalanceEvents(crossDataHistPOList); // 获取失衡事件集合
        vo.setUnbalanceTimes(unbalanceEventList.size());
        int unbalanceSum = unbalanceEventList.stream()
                .filter(Objects::nonNull)
                .map(CrossDataHistPO::getDuration)
                .filter(Objects::nonNull)
                .mapToInt(Integer::intValue)
                .sum();
        vo.setUnbalanceSum(unbalanceSum);

        List<CrossDataHistPO> spilloverEventList = buildSpilloverEvents(crossDataHistPOList);  // 获取溢出事件集合
        vo.setSpilloverTimes(spilloverEventList.size());
        int spilloverSum = spilloverEventList.stream()
                .filter(Objects::nonNull)
                .map(CrossDataHistPO::getDuration)
                .filter(Objects::nonNull)
                .mapToInt(Integer::intValue)
                .sum();
        vo.setSpilloverSum(spilloverSum);

        // 相位空放指标
        Date startDate = bo.getStartDate();
        Date endDate = bo.getEndDate();
        if (isZeros(endDate)) {
            endDate = DateUtil.offsetDay(endDate, 1); // 包含最后一天
        }
        long startTime = startDate.getTime();
        long endTime = endDate.getTime();
        KafkaConsumerUtil kafkaConsumerUtil = new KafkaConsumerUtil(bootstrapServers, "empty-phase-evaluate");
        List<PhaseEmptyResult> phaseEmptyResults =
                kafkaConsumerUtil.consumeEmptyPhaseForTimeRange(emptyPhaseTopic, 0, startTime, endTime);

        int emptyPhaseCount = (int) phaseEmptyResults.stream()
                .filter(result -> result != null && crossId.equals(result.getCrossId()))
                .count();
        int emptyPhaseSum = phaseEmptyResults.stream()
                .filter(result -> result != null && crossId.equals(result.getCrossId()))
                .map(PhaseEmptyResult::getDuration)
                .filter(Objects::nonNull)
                .mapToInt(Integer::intValue)
                .sum();
        vo.setEmptyPhaseTimes(emptyPhaseCount);
        vo.setEmptyPhaseSum(emptyPhaseSum);

        Integer schemeProblems = calcSchemeProblems(
                congestionEventList, unbalanceEventList, spilloverEventList, crossId); // 计算方案问题
        vo.setSchemeProblems(schemeProblems);

        return vo;
    }

    private boolean isInTimeRange(Integer batchTime) {
        // 早晚高峰及平峰时段
        int hour = (batchTime % 86400) / 3600;
        int minute = (batchTime % 3600) / 60;
        return (hour >= 7 && hour < 9) ||
                (hour >= 16 && (hour < 18 || (hour == 18 && minute < 30))) ||
                (hour >= 11 && hour < 13);
    }

    private List<CrossDataHistPO> buildCrossDataHistPOList(CrossIdAndStartEndDateBO bo, String crossId) {
        Date startDate = bo.getStartDate();
        Date endDate = bo.getEndDate();
        if (isZeros(endDate)) {
            endDate = DateUtil.offsetDay(endDate, 1); // 包含最后一天
        }
        int startStamp = (int) (startDate.getTime() / 1000); // 10位时间戳
        int endStamp = (int) (endDate.getTime() / 1000);
        List<CrossDataHistPO> crossDataHistPOList = crossDataHistMapper.selectByCrossIdAndStartEnd(
                crossId, startStamp, endStamp);
        return crossDataHistPOList;
    }

    private boolean isZeros(Date endDate) {
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(endDate);
        return calendar.get(Calendar.HOUR_OF_DAY) == 0 &&
                calendar.get(Calendar.MINUTE) == 0 &&
                calendar.get(Calendar.SECOND) == 0 &&
                calendar.get(Calendar.MILLISECOND) == 0;
    }

    @Override
    public RunningEvaluateStatusVO congestionStatus(CrossIdAndStartEndDateBO bo) {
        return evaluateStatus(bo, CrossStatusEnum.CONGESTION);
    }

    @Override
    public RunningEvaluateStatusVO unbalanceStatus(CrossIdAndStartEndDateBO bo) {
        return evaluateStatus(bo, CrossStatusEnum.UNBALANCE);
    }

    @Override
    public RunningEvaluateStatusVO spilloverStatus(CrossIdAndStartEndDateBO bo) {
        return evaluateStatus(bo, CrossStatusEnum.SPILLOVER);
    }

    public RunningEvaluateStatusVO evaluateStatus(CrossIdAndStartEndDateBO bo, CrossStatusEnum statusEnum) {
        RunningEvaluateStatusVO runningEvaluateStatusVO = new RunningEvaluateStatusVO();
        String crossId = bo.getCrossId();
        int status = statusEnum.getCode();
        // 构造入参开始和结束时间戳
        Date startDate = bo.getStartDate();
        Date endDate = bo.getEndDate();
        endDate = DateUtil.offsetDay(endDate, 1); // 包含最后一天
        int startStamp = (int) (startDate.getTime() / 1000); // 10位时间戳
        int endStamp = (int) (endDate.getTime() / 1000);

        runningEvaluateStatusVO.setRunningEvaluateIndexStatusVOList(buildRes(bo, crossId, status));
        // 获取问题列表
        // 路口级别全量数据
        List<MetricHistDTO> crossDTOList = crossDataHistMapper.selectMetricHistDTO(
                crossId, startStamp, endStamp);
        // 过滤有问题的记录
        List<MetricHistDTO> filteredList = crossDTOList.stream()
                .filter(metricHistDTO -> metricHistDTO.getStatus() != null && metricHistDTO.getStatus().equals(status))
                .collect(Collectors.toList());

        runningEvaluateStatusVO.setProblemStatusList(buildProblemStatusList(filteredList));
        runningEvaluateStatusVO.setTimeList(TimeArrayUtil.getMinuteSectionArray(5));

        return runningEvaluateStatusVO;
    }

    @Override
    public List<RunningEvaluateSchemeProblemsVO> schemeProblems(CrossIdAndStartEndDateBO bo) {
        String crossId = bo.getCrossId();
        List<RunningEvaluateSchemeProblemsVO> res = new ArrayList<>();
        List<String> timeArray = TimeArrayUtil.getHourArray();

        for (String time : timeArray) {
            RunningEvaluateSchemeProblemsVO vo = new RunningEvaluateSchemeProblemsVO();
            vo.setTime(time);
            List<CrossDataHistPO> crossDataHistPOList = buildCrossDataHistPOList(bo, crossId);
            // 找出有问题的记录
            List<CrossDataHistPO> collect = crossDataHistPOList.stream()
                    .filter(po -> !Objects.equals(po.getStatus(), CrossStatusEnum.NORMAL.getCode()))
                    .collect(Collectors.toList());
            // 按采集时间小时进行聚合
            Map<String, List<CrossDataHistPO>> timePOMap = buildTimePOMap(collect);
            ;
            // 找出当前小时的记录
            List<CrossDataHistPO> currentHourList = timePOMap.get(time.substring(0, 2)); // 保留小时字符串
            if (currentHourList != null) {
                // 按相同开始时间去重，保留持续时间最长的一个
                List<CrossDataHistPO> uniqueList = currentHourList.stream()
                        .collect(Collectors.groupingBy(
                                CrossDataHistPO::getStartTime, // 按startTime分组
                                Collectors.collectingAndThen(
                                        // 在每个组中找到duration最大的元素
                                        Collectors.maxBy(Comparator.comparingLong(CrossDataHistPO::getDuration)),
                                        Optional::get // 从Optional中获取元素
                                )
                        ))
                        .values().stream().collect(Collectors.toList()); // 从Map中获取值并收集到新的列表中
                // 统计数量
                vo.setCounts(uniqueList.size());
                // 找出出现问题的日期和方案
                List<RunningEvaluateSchemeProblemsVO.DateAndScheme> dateAndSchemeList = new ArrayList<>();
                dateAndSchemeList = buildDateAndSchemeList(uniqueList, crossId);
                vo.setDateAndSchemeList(dateAndSchemeList);
            }

            res.add(vo);
        }
        return res;
    }

    private Map<String, List<CrossDataHistPO>> buildTimePOMap(List<CrossDataHistPO> crossDataHistPOList) {
        SimpleDateFormat sdf = new SimpleDateFormat("HH");
        Map<String, List<CrossDataHistPO>> timePOMap = crossDataHistPOList.stream()
                .collect(Collectors.groupingBy(po -> {
                    Date date = new Date((long) po.getBatchTime() * 1000); // 将10位时间戳转换为Date对象
                    return sdf.format(date); // 将Date对象转换为"HH"格式的字符串
                }));
        return timePOMap;
    }

    @Override
    public RunningEvaluateMetricsDetailVO metricsDetail(MetricsDetailBO bo) {
        String crossId = bo.getCrossId();
        Integer dir = bo.getDir();
        String turn = bo.getTurn();
        Integer laneSort = bo.getLaneSort();
        // 构造某一天开始和结束时间戳
        Date date = bo.getDate();
        Date startDate = date;
        Date endDate = DateUtil.offsetDay(date, 1); // 包含最后一天
        int startStamp = (int) (startDate.getTime() / 1000); // 10位时间戳
        int endStamp = (int) (endDate.getTime() / 1000);

        Integer minutes = bo.getMinutes();

        List<MetricHistDTO> metricHistDTOList = new ArrayList<>();
        if (ObjectUtil.isNotEmpty(laneSort)) {
            Integer turnInt = TurnConvertEnum.getKeyByCode(turn);
            String laneId = laneInfoMapper.selectIdByCrossIdDirTurn2(crossId, dir, turnInt);
            metricHistDTOList = crossLaneDataHistMapper.selectMetricHistDTO(
                    crossId, startStamp, endStamp, laneId);
        } else if (ObjectUtil.isNotEmpty(turn)) {
            metricHistDTOList = crossTurnDataHistMapper.selectMetricHistDTO(
                    crossId, startStamp, endStamp, dir, turn);
        } else if (ObjectUtil.isNotEmpty(dir)) {
            metricHistDTOList = crossDirDataHistMapper.selectMetricHistDTO(
                    crossId, startStamp, endStamp, dir);
        } else { // 路口级别
            metricHistDTOList = crossDataHistMapper.selectMetricHistDTO(
                    crossId, startStamp, endStamp);
            // 相位空放指标
            long startTime = startDate.getTime();
            long endTime = endDate.getTime();
            KafkaConsumerUtil kafkaConsumerUtil = new KafkaConsumerUtil(bootstrapServers, "empty-phase-comsumer");
            List<PhaseEmptyResult> phaseEmptyResults =
                    kafkaConsumerUtil.consumeEmptyPhaseForTimeRange(emptyPhaseTopic, 0, startTime, endTime);
            fillPhaseEmpty(metricHistDTOList, crossId, phaseEmptyResults);
        }

        RunningEvaluateMetricsDetailVO res = new RunningEvaluateMetricsDetailVO();
        // 路口级别全量数据
        List<MetricHistDTO> crossDTOList = crossDataHistMapper.selectMetricHistDTO(
                crossId, startStamp, endStamp);
        // 过滤有问题的记录
        List<MetricHistDTO> filteredList = crossDTOList.stream()
                .filter(metricHistDTO -> metricHistDTO.getStatus() != null && metricHistDTO.getStatus() != 0)
                .collect(Collectors.toList());
        if (CollectionUtil.isNotEmpty(filteredList)) {
            res.setProblemStatusList(buildProblemStatusList(filteredList));
            res.setSchemeList(buildSchemeList(filteredList, crossId));
        }
        if (CollectionUtil.isEmpty(metricHistDTOList)) {
            return res;
        }
        // 使用全量数据按时间粒度聚合指标
        List<RunningEvaluateMetricsDetailVO.CrossMetrics> crossMetrics = buildMetricsList(metricHistDTOList, minutes);
        res.setMetricsList(crossMetrics);
        return res;
    }

    private void fillPhaseEmpty(List<MetricHistDTO> metricHistDTOList, String crossId,
                                List<PhaseEmptyResult> phaseEmptyResults) {

        List<PhaseEmptyResult> crossResults = phaseEmptyResults.stream()
                .filter(result -> crossId.equals(result.getCrossId()))
                .collect(Collectors.toList());

        for (MetricHistDTO metricHistDTO : metricHistDTOList) {
            // 计算结束时间
            Date endTime = new Date(metricHistDTO.getStartTime().getTime() + 5 * 60 * 1000);  // 增加5分钟

            // 遍历每个 PhaseEmptyResult 来查找和累加 duration
            for (PhaseEmptyResult phaseEmptyResult : crossResults) {
                if (phaseEmptyResult.getDetectTime() * 1000 >= metricHistDTO.getStartTime().getTime()
                        && phaseEmptyResult.getDetectTime() * 1000 <= endTime.getTime()) {
                    // 累加 duration
                    metricHistDTO.setEmptyPhase(metricHistDTO.getEmptyPhase() + phaseEmptyResult.getDuration());
                }
            }
        }

    }

    public static List<RunningEvaluateMetricsDetailVO.CrossMetrics> buildMetricsList(
            List<MetricHistDTO> metricHistDTOList, Integer minutes) {

        // 按时间段分组
        Map<String, List<MetricHistDTO>> groupedByTime = metricHistDTOList.stream()
                .collect(Collectors.groupingBy(metricHistDTO -> {
                    long timestampInMillSeconds = metricHistDTO.getBatchTime() * 1000L;
                    Date date = new Date(timestampInMillSeconds);
                    SimpleDateFormat sdf = new SimpleDateFormat("HH:mm");
                    int minuteOfDay = date.getMinutes() + date.getHours() * 60;
                    int roundedMinute = (minuteOfDay / minutes) * minutes;
                    date.setHours(roundedMinute / 60);
                    date.setMinutes(roundedMinute % 60);
                    return sdf.format(date);
                }));

        // 获取时间段数组
        List<String> minuteSectionArray = TimeArrayUtil.getMinuteSectionArray(minutes);
        // 计算指标
        List<RunningEvaluateMetricsDetailVO.CrossMetrics> res = new ArrayList<>();
        for (String section : minuteSectionArray) {
            RunningEvaluateMetricsDetailVO.CrossMetrics crossMetrics = new RunningEvaluateMetricsDetailVO.CrossMetrics();
            List<MetricHistDTO> dtoList = groupedByTime.get(section);
            if (CollectionUtil.isNotEmpty(dtoList)) {
                int flowSum = 0;
                double speedSum = 0.0;
                int capacitySum = 0;
                double sturationSum = 0.0;
                double stopTimesSum = 0.0;
                int delayTimeSum = 0;
                double effusionRateSum = 0.0;
                int emptyPhaseSum = 0;
                for (MetricHistDTO metricHistDTO : dtoList) {
                    Integer flow = metricHistDTO.getFlow();
                    Double speed = metricHistDTO.getSpeed();
                    Integer capacity = metricHistDTO.getCapacity();
                    Double sturation = metricHistDTO.getSturation();
                    Double stopTimes = metricHistDTO.getStopTimes();
                    Integer delayTime = metricHistDTO.getDelayTime();
                    Double effusionRate = metricHistDTO.getEffusionRate();
                    Integer emptyPhase = metricHistDTO.getEmptyPhase();
                    if (flow != null) flowSum += flow;
                    if (speed != null) speedSum += speed;
                    if (capacity != null) capacitySum += capacity;
                    if (sturation != null) sturationSum += sturation;
                    if (stopTimes != null) stopTimesSum += stopTimes;
                    if (delayTime != null) delayTimeSum += delayTime;
                    if (effusionRate != null) effusionRateSum += effusionRate;
                    if (emptyPhase != null) emptyPhaseSum += emptyPhase;
                }
                int size = dtoList.size();
                crossMetrics.setFlow(flowSum);
                crossMetrics.setSpeed(speedSum / size);
                crossMetrics.setCapacity(capacitySum / size);
                crossMetrics.setSturation(sturationSum / size);
                crossMetrics.setStopTimes(stopTimesSum / size);
                crossMetrics.setDelayTime(delayTimeSum / size);
                crossMetrics.setEffusionRate(effusionRateSum / size);
                crossMetrics.setEmptyPhase(emptyPhaseSum);
                Date currentDate = dtoList.get(0).getStartTime();
                Calendar instance = Calendar.getInstance();
                instance.setTime(currentDate);
                instance.set(Calendar.HOUR_OF_DAY, Integer.parseInt(section.substring(0, 2)));
                instance.set(Calendar.MINUTE, 0);
                crossMetrics.setTimeStamp(instance.getTime());
                crossMetrics.setMetricTime(section);
            }
            if (StringUtils.isNotBlank(crossMetrics.getMetricTime())) {
                res.add(crossMetrics);
            }
        }
        return res;
    }

    private List<RunningEvaluateMetricsDetailVO.ProblemScheme> buildSchemeList(
            List<MetricHistDTO> metricHistDTOList, String crossId) {
        List<BaseCrossSchemePO> baseCrossSchemePOList = baseCrossSchemeMapper.selectAll();
        List<RunningEvaluateMetricsDetailVO.ProblemScheme> res = new ArrayList<>();
        // 验证数据是否正确
        if (!isDataValid(metricHistDTOList)) {
            return res;
        }
        // 根据持续时长去重
        List<MetricHistDTO> uniqueList = metricHistDTOList.stream()
                .collect(Collectors.groupingBy(
                        MetricHistDTO::getStartTime, // 按startTime分组
                        Collectors.collectingAndThen(
                                // 在每个组中找到duration最大的元素
                                Collectors.maxBy(Comparator.comparingLong(MetricHistDTO::getDuration)),
                                Optional::get // 从Optional中获取元素
                        )
                ))
                .values().stream().collect(Collectors.toList()); // 从Map中获取值并收集到新的列表中

        for (MetricHistDTO dto : uniqueList) {
            Date abnormalEventStartTime = dto.getStartTime();
            Integer duration = dto.getDuration();
            Date abnormalEventEndTime = DateUtil.offsetMinute(abnormalEventStartTime, duration);
            String eventStartTime = HOUR_SDF.format(abnormalEventStartTime);
            String eventEndTime = HOUR_SDF.format(abnormalEventEndTime);

            Date datetime = new Date();
            String dateStr = DAY_SDF.format(datetime);
            // 获取当前计划ID
            Integer planId = findPlanId(datetime, dateStr, crossId);
            List<CrossSectionPO> crossSectionPOList = baseCrossSectionMapper.selectByCrossAndPlan(crossId, planId);
            for (CrossSectionPO crossSectionPO : crossSectionPOList) {
                String sectionStartTime = crossSectionPO.getStartTime();
                String sectionEndTime = crossSectionPO.getEndTime();
                // 比较两组时间段是否相交
                boolean isIntersecting = isTimeIntersecting(
                        eventStartTime, eventEndTime, sectionStartTime, sectionEndTime);
                buildResIfIntersecting(res, crossSectionPO, sectionStartTime, sectionEndTime, isIntersecting,
                        baseCrossSchemePOList);
            }
        }
        // 去除相同方案
        List<RunningEvaluateMetricsDetailVO.ProblemScheme> uniqueRes = res.stream()
                .distinct()
                .collect(Collectors.toList());
        return uniqueRes;
    }

    void buildResIfIntersecting(List<RunningEvaluateMetricsDetailVO.ProblemScheme> res, CrossSectionPO crossSectionPO,
                                String sectionStartTime, String sectionEndTime, boolean isIntersecting,
                                List<BaseCrossSchemePO> crossSchemePOList) {
        if (isIntersecting) {
            Integer schemeId = crossSectionPO.getSchemeId();

            Optional<BaseCrossSchemePO> result = crossSchemePOList.stream()
                    .filter(crossScheme -> schemeId.equals(crossScheme.getId()))
                    .findFirst();
            BaseCrossSchemePO baseCrossSchemePO = result.get();
            String schemeName = baseCrossSchemePO.getName();

            RunningEvaluateMetricsDetailVO.ProblemScheme problemScheme =
                    new RunningEvaluateMetricsDetailVO.ProblemScheme();
            problemScheme.setSchemeName(schemeName);
            problemScheme.setSchemeId(schemeId);
            problemScheme.setStartTime(sectionStartTime);
            problemScheme.setEndTime(sectionEndTime);
            res.add(problemScheme);
        }
    }

    List<RunningEvaluateMetricsDetailVO.ProblemStatus> buildProblemStatusList(
            List<MetricHistDTO> metricHistDTOList) {
        List<RunningEvaluateMetricsDetailVO.ProblemStatus> res = new ArrayList<>();
        // 验证数据是否正确
        if (!isDataValid(metricHistDTOList)) {
            return res;
        }
        // 根据持续时长去重
        List<MetricHistDTO> uniqueList = metricHistDTOList.stream()
                .collect(Collectors.groupingBy(
                        MetricHistDTO::getStartTime, // 按startTime分组
                        Collectors.collectingAndThen(
                                // 在每个组中找到duration最大的元素
                                Collectors.maxBy(Comparator.comparingLong(MetricHistDTO::getDuration)),
                                Optional::get // 从Optional中获取元素
                        )
                ))
                .values().stream().collect(Collectors.toList()); // 从Map中获取值并收集到新的列表中

        for (MetricHistDTO metricHistDTO : uniqueList) {
            RunningEvaluateMetricsDetailVO.ProblemStatus problemStatus =
                    new RunningEvaluateMetricsDetailVO.ProblemStatus();
            Integer status1 = metricHistDTO.getStatus();
            problemStatus.setStatus(status1);
            Date startTime1 = metricHistDTO.getStartTime();
            problemStatus.setStartTime(startTime1);
            Integer duration1 = metricHistDTO.getDuration();
            DateTime endTime = DateUtil.offsetMinute(startTime1, duration1);
            problemStatus.setEndTime(endTime);
            problemStatus.setDurationMinutes(duration1);
            res.add(problemStatus);
        }

        // 合并时间区间
        // 先按 status 排序，status 相同再按 startTime 排序
        List<RunningEvaluateMetricsDetailVO.ProblemStatus> sortedList = res.stream()
                .sorted(Comparator.comparingInt(RunningEvaluateMetricsDetailVO.ProblemStatus::getStatus)
                        .thenComparing(RunningEvaluateMetricsDetailVO.ProblemStatus::getStartTime))
                .collect(Collectors.toList());

        List<RunningEvaluateMetricsDetailVO.ProblemStatus> mergedList = new ArrayList<>();
        RunningEvaluateMetricsDetailVO.ProblemStatus current = sortedList.get(0);

        for (int i = 1; i < sortedList.size(); i++) {
            RunningEvaluateMetricsDetailVO.ProblemStatus next = sortedList.get(i);
            if (Objects.equals(current.getStatus(), next.getStatus())
                    && !current.getEndTime().before(next.getStartTime())) {
                // 如果有交叉，则合并记录
                current.setEndTime(new Date(Math.max(current.getEndTime().getTime(), next.getEndTime().getTime())));
                current.setDurationMinutes(
                        (int) ((current.getEndTime().getTime() - current.getStartTime().getTime()) / (60 * 1000)));
            } else {
                // 否则，将当前记录添加到结果列表中，并更新当前记录
                mergedList.add(current);
                current = next;
            }
        }
        mergedList.add(current); // 添加最后一个记录

        return mergedList;
    }

    private boolean isDataValid(List<MetricHistDTO> metricHistDTOList) {
        if (CollectionUtil.isEmpty(metricHistDTOList)) {
            return false;
        }
        MetricHistDTO dto = metricHistDTOList.get(0);
        Integer status = dto.getStatus();
        Date startTime = dto.getStartTime();
        Integer duration = dto.getDuration();
        if (ObjectUtil.isEmpty(startTime) || ObjectUtil.isEmpty(duration) || ObjectUtil.isEmpty(status)) {
            return false;
        }
        return true;
    }

    @Override
    public RunningEvaluateScopeTreeVO scopeTree(CrossIdBO bo) {
        String crossId = bo.getCrossId();
        RunningEvaluateScopeTreeVO runningEvaluateScopeTreeVO = new RunningEvaluateScopeTreeVO();
        runningEvaluateScopeTreeVO.setCrossId(crossId);
        List<RunningEvaluateScopeTreeVO.DirVO> dirVOList = buildDirVOList(crossId);
        runningEvaluateScopeTreeVO.setChild(dirVOList);
        return runningEvaluateScopeTreeVO;
    }

    @Override
    public List<RunningEvaluateHeatMapVO> heatMap(HeatMapBO bo) {
        String crossId = bo.getCrossId();
        Date startDate = bo.getStartDate();
        Date endDate = bo.getEndDate();
        endDate = DateUtil.offsetDay(endDate, 1); // 包含最后一天
        int startStamp = (int) (startDate.getTime() / 1000); // 10位时间戳
        int endStamp = (int) (endDate.getTime() / 1000);
        Integer status = bo.getStatus();

        List<RunningEvaluateHeatMapVO> res = new ArrayList<>();
        List<CrossDataHistPO> crossDataHistPOList = crossDataHistMapper.selectByCrossIdAndStartEnd(
                crossId, startStamp, endStamp);
        // 根据 status 值选择分组字段
        Function<CrossDataHistPO, String> groupByField;
        switch (status) {
            case 1:
                groupByField = CrossDataHistPO::getUnbalanceDirs;
                break;
            case 2:
                groupByField = CrossDataHistPO::getCongestionDirs;
                break;
            case 3:
                groupByField = CrossDataHistPO::getSpilloverDirs;
                break;
            default:
                throw new IllegalArgumentException("状态值非法");
        }
        // 过滤 status 字段值为 status 变量的对象，并按照方向分组
        Map<String, List<CrossDataHistPO>> grouped = crossDataHistPOList.stream()
                .flatMap(po -> {
                    String dirs = "";
                    if (status == 1) {
                        dirs = po.getUnbalanceDirs();
                    } else if (status == 2) {
                        dirs = po.getCongestionDirs();
                    } else if (status == 3) {
                        dirs = po.getSpilloverDirs();
                    }
                    return Arrays.stream(dirs.split(","))
                            .map(dir -> new AbstractMap.SimpleEntry<>(dir, po));
                })
                .collect(Collectors.groupingBy(
                        Map.Entry::getKey,
                        Collectors.mapping(Map.Entry::getValue, Collectors.toList())
                ));
        // 对每个组进行处理，仅保留 start_time 相同的记录中 duration 最大的一条
        Map<String, List<CrossDataHistPO>> filteredGrouped = grouped.entrySet().stream()
                .collect(Collectors.toMap(
                        Map.Entry::getKey,
                        entry -> entry.getValue().stream()
                                .collect(Collectors.groupingBy(CrossDataHistPO::getStartTime))
                                .values().stream()
                                .map(list -> list.stream().max(Comparator.comparingInt(CrossDataHistPO::getDuration))
                                        .orElse(null))
                                .collect(Collectors.toList())
                ));

        // 遍历 filteredGrouped，提取每个 key 和与该 key 关联的列表的元素个数
        filteredGrouped.forEach((key, list) -> {
            RunningEvaluateHeatMapVO runningEvaluateHeatMapVO = new RunningEvaluateHeatMapVO();
            if (ObjectUtil.isNotEmpty(key)) {
                runningEvaluateHeatMapVO.setDir(Integer.parseInt(key));
                runningEvaluateHeatMapVO.setCounts(list.size());
                res.add(runningEvaluateHeatMapVO);
            }
        });

        return res;
    }

    private List<RunningEvaluateScopeTreeVO.DirVO> buildDirVOList(String crossId) {
        List<RunningEvaluateScopeTreeVO.DirVO> res = new ArrayList<>();
        Integer type = 1; // 进口
        List<BaseCrossDirInfoPO> baseCrossDirInfoPOList =
                baseCrossDirInfoMapper.selectByCrossIdAndInOutType(crossId, type);

        for (BaseCrossDirInfoPO baseCrossDirInfoPO : baseCrossDirInfoPOList) {
            RunningEvaluateScopeTreeVO.DirVO dirVO = new RunningEvaluateScopeTreeVO.DirVO();
            Integer dir = baseCrossDirInfoPO.getDirType();
            dirVO.setDir(dir);
            List<RunningEvaluateScopeTreeVO.TurnVO> turnVOList = buildTurnVOList(crossId, dir);
            dirVO.setChild(turnVOList);
            res.add(dirVO);
        }

        return res;
    }

    private List<RunningEvaluateScopeTreeVO.TurnVO> buildTurnVOList(String crossId, Integer dir) {
        List<RunningEvaluateScopeTreeVO.TurnVO> res = new ArrayList<>();
        List<CrossTurnInfoPO> crossTurnInfoPOList = baseCrossTurnInfoMapper.selectByCrossIdAndDir(crossId, dir);

        for (CrossTurnInfoPO crossTurnInfoPO : crossTurnInfoPOList) {
            RunningEvaluateScopeTreeVO.TurnVO turnVO = new RunningEvaluateScopeTreeVO.TurnVO();
            String turnType = crossTurnInfoPO.getTurnType();
            turnVO.setTurn(turnType);
            List<RunningEvaluateScopeTreeVO.LaneVO> laneVOList = buildLaneVOList(crossId, dir, turnType);
            turnVO.setChild(laneVOList);
            res.add(turnVO);
        }

        return res;
    }

    private List<RunningEvaluateScopeTreeVO.LaneVO> buildLaneVOList(String crossId, Integer dir, String turnType) {
        List<RunningEvaluateScopeTreeVO.LaneVO> res = new ArrayList<>();
        List<Integer> keyList = TurnConvertEnum.getKeyListBySingleCode(turnType);
        Integer type = 2; // 进口
        if (ObjectUtil.isNotEmpty(keyList)) {
            List<LaneInfoPO> laneInfoPOList = laneInfoMapper.selectByTurnType(crossId, type, dir, keyList);
            for (LaneInfoPO laneInfoPO : laneInfoPOList) {
                RunningEvaluateScopeTreeVO.LaneVO laneVO = new RunningEvaluateScopeTreeVO.LaneVO();
                Integer sort = laneInfoPO.getSort();
                laneVO.setLaneSort(sort);
                res.add(laneVO);
            }
        }
        return res;
    }

    private List<RunningEvaluateSchemeProblemsVO.DateAndScheme> buildDateAndSchemeList(
            List<CrossDataHistPO> uniqueList, String crossId) {
        List<RunningEvaluateSchemeProblemsVO.DateAndScheme> res = new ArrayList<>();
        // 将问题按日期分组
        Map<LocalDate, List<CrossDataHistPO>> groupedByDate = uniqueList.stream()
                .collect(Collectors.groupingBy(
                        // 将10位时间戳转换为Instant
                        crossDataHistPO -> Instant.ofEpochSecond(crossDataHistPO.getBatchTime())
                                .atZone(ZoneId.systemDefault()) // 将Instant转换为系统默认时区的ZonedDateTime
                                .toLocalDate() // 获取LocalDate
                ));

        for (Map.Entry<LocalDate, List<CrossDataHistPO>> entry : groupedByDate.entrySet()) {
            LocalDate localDate = entry.getKey(); // 获取日期
            List<CrossDataHistPO> crossDataHistPOs = entry.getValue(); // 获取该日期下的CrossDataHistPO对象列表

            RunningEvaluateSchemeProblemsVO.DateAndScheme dateAndScheme =
                    new RunningEvaluateSchemeProblemsVO.DateAndScheme();
            Date date = Date.from(localDate.atStartOfDay()
                    .atZone(ZoneId.systemDefault())
                    .toInstant());
            dateAndScheme.setProblemDate(date);
            Set<String> schemeNameSet = new HashSet<>();

            for (CrossDataHistPO abnormalEvent : crossDataHistPOs) {
                SimpleDateFormat sdf = new SimpleDateFormat("HH:mm");
                Date abnormalEventStartTime = abnormalEvent.getStartTime();
                Integer duration = abnormalEvent.getDuration();
                Date abnormalEventEndTime = DateUtil.offsetMinute(abnormalEventStartTime, duration);
                String eventStartTime = sdf.format(abnormalEventStartTime);
                String eventEndTime = sdf.format(abnormalEventEndTime);

                SimpleDateFormat sdf2 = new SimpleDateFormat("yyyy-MM-dd");
                Date datetime = new Date();
                String dateStr = sdf2.format(datetime);
                // 获取当前计划ID
                Integer planId = findPlanId(datetime, dateStr, crossId);
                List<CrossSectionPO> crossSectionPOList = baseCrossSectionMapper.selectByCrossAndPlan(crossId, planId);
                for (CrossSectionPO crossSectionPO : crossSectionPOList) {
                    String sectionStartTime = crossSectionPO.getStartTime();
                    String sectionEndTime = crossSectionPO.getEndTime();
                    // 比较两组时间段是否相交
                    boolean isIntersecting = isTimeIntersecting(
                            eventStartTime, eventEndTime, sectionStartTime, sectionEndTime);
                    if (isIntersecting) {
                        Integer schemeId = crossSectionPO.getSchemeId();
                        BaseCrossSchemePO baseCrossSchemePO = baseCrossSchemeMapper.selectById(schemeId);
                        schemeNameSet.add(baseCrossSchemePO.getName());
                        dateAndScheme.setSchemeNameSet(schemeNameSet);

                    }
                }
            }
            res.add(dateAndScheme);
        }
        return res;
    }

    @NotNull
    private List<RunningEvaluateIndexStatusVO> buildRes(CrossIdAndStartEndDateBO bo, String crossId, int status) {
        List<CrossDataHistPO> crossDataHistPOList = buildCrossDataHistPOList(bo, crossId);
        List<RunningEvaluateIndexStatusVO> res = buildRunningEvaluateIndexStatusVOList(crossDataHistPOList, status);
        return res;
    }

    @NotNull
    private List<RunningEvaluateIndexStatusVO> buildRunningEvaluateIndexStatusVOList(
            List<CrossDataHistPO> crossDataHistPOList, int status) {
        Map<String, List<CrossDataHistPO>> timePOMap = buildTimePOMap(crossDataHistPOList);

        // 构造结果集
        List<RunningEvaluateIndexStatusVO> res = new ArrayList<>();
        List<String> timeArray = TimeArrayUtil.getHourArray();
        for (String time : timeArray) {
            RunningEvaluateIndexStatusVO vo = new RunningEvaluateIndexStatusVO();
            vo.setTime(time);
            List<CrossDataHistPO> subList = timePOMap.get(time.substring(0, 2)); // 保留小时字符串
            if (subList != null) {
                Double avgCongestionIndex = calcAvgIndex(subList, status);
                vo.setIndex(avgCongestionIndex);
            }

            res.add(vo);
        }
        return res;
    }

    private Double calcAvgIndex(List<CrossDataHistPO> subList, int status) {
        Double average = 0.0;

        if (status == CrossStatusEnum.CONGESTION.getCode()) {
            average = subList.stream()
                    .mapToDouble(CrossDataHistPO::getCongestionIndex)
                    .average()
                    .orElse(0.0);
        } else if (status == CrossStatusEnum.UNBALANCE.getCode()) {
            average = subList.stream()
                    .mapToDouble(CrossDataHistPO::getUnbalanceIndex)
                    .average()
                    .orElse(0.0);
        } else if (status == CrossStatusEnum.SPILLOVER.getCode()) {
            average = subList.stream()
                    .mapToDouble(CrossDataHistPO::getSpilloverIndex)
                    .average()
                    .orElse(0.0);
        }

        DecimalFormat df = new DecimalFormat("#.00");
        String formattedAverage = df.format(average);
        return Double.parseDouble(formattedAverage);
    }

    private Integer calcSchemeProblems(List<CrossDataHistPO> congestionEventList,
                                       List<CrossDataHistPO> unbalanceEventList,
                                       List<CrossDataHistPO> spilloverEventList,
                                       String crossId) {

        Set<Integer> schemeIdSet = new HashSet<>(); // 用于方案ID去重

        List<CrossDataHistPO> allList = Stream.of(congestionEventList, unbalanceEventList, spilloverEventList)
                .flatMap(List::stream)
                .collect(Collectors.toList());

        for (CrossDataHistPO abnormalEvent : allList) {
            Date abnormalEventStartTime = abnormalEvent.getStartTime();
            Integer duration = abnormalEvent.getDuration();
            Date abnormalEventEndTime = DateUtil.offsetMinute(abnormalEventStartTime, duration);
            String eventStartTime = HOUR_SDF.format(abnormalEventStartTime);
            String eventEndTime = HOUR_SDF.format(abnormalEventEndTime);

            Date datetime = new Date();
            String dateStr = DAY_SDF.format(datetime);
            // 获取当前计划ID
            Integer planId = findPlanId(datetime, dateStr, crossId);
            List<CrossSectionPO> crossSectionPOList = baseCrossSectionMapper.selectByCrossAndPlan(crossId, planId);
            for (CrossSectionPO crossSectionPO : crossSectionPOList) {
                String sectionStartTime = crossSectionPO.getStartTime();
                String sectionEndTime = crossSectionPO.getEndTime();
                // 比较两组时间段是否相交
                boolean isIntersecting = isTimeIntersecting(
                        eventStartTime, eventEndTime, sectionStartTime, sectionEndTime);
                if (isIntersecting) {
                    Integer schemeId = crossSectionPO.getSchemeId();
                    schemeIdSet.add(schemeId);
                }
            }
        }

        return schemeIdSet.size();
    }

    Integer findPlanId(Date datetime, String dateStr, String crossId) {
        CrossIdBO crossIdBO = new CrossIdBO();
        crossIdBO.setCrossId(crossId);
        RunningPlanDTO runningPlanDTO = listRunningPlan(crossIdBO);
        List<RunningPlanDTO.SchedulesPlanListElement> schedulesPlanList = runningPlanDTO.getSchedulesPlanList();
        for (RunningPlanDTO.SchedulesPlanListElement schedulesPlan : schedulesPlanList) {
            List<RunningPlanDTO.ExecListElement> execList = schedulesPlan.getExecList();
            for (RunningPlanDTO.ExecListElement execListElement : execList) {
                // 先查询特殊日期，再查询星期
                List<String> specialDateList = execListElement.getSpecialDateList();
                List<Integer> week = execListElement.getWeek();
                if (CollectionUtil.isNotEmpty(specialDateList)) {
                    for (String specialDate : specialDateList) {
                        String substring = specialDate.substring(0, 10);
                        if (Objects.equals(dateStr, substring)) {
                            Integer planId = execListElement.getPlanId();
                            return planId;
                        }
                    }
                }
                if (CollectionUtil.isNotEmpty(week)) {
                    for (Integer day : week) {
                        // 1表示周日，2表示周一
                        int today = DateUtil.dayOfWeek(datetime);
                        today -= 1;
                        if (today == 0) {
                            today = 7;
                        }
                        if (day == today) {
                            Integer planId = execListElement.getPlanId();
                            return planId;
                        }
                    }
                }
            }
        }
        return null;
    }

    public RunningPlanDTO listRunningPlan(CrossIdBO crossIdBO) {
        RunningPlanDTO runningPlanDTO = new RunningPlanDTO();
        String crossId = crossIdBO.getCrossId();
        runningPlanDTO.setCrossId(crossId);
        // 构造dailyPlanList
        List<RunningPlanDTO.DailyPlanListElement> dailyPlanList = buildDailyPlanList(crossId);
        runningPlanDTO.setDailyPlanList(dailyPlanList);
        // 构造schedulesPlanList
        List<RunningPlanDTO.SchedulesPlanListElement> schedulesPlanList = buildSchedulesPlanList(crossId);
        runningPlanDTO.setSchedulesPlanList(schedulesPlanList);

        return runningPlanDTO;
    }

    private List<RunningPlanDTO.DailyPlanListElement> buildDailyPlanList(String crossId) {
        List<RunningPlanDTO.DailyPlanListElement> dailyPlanList = new ArrayList<>();
        dailyPlanList = baseCrossPlanMapper.selectDailyPlanList(crossId);
        if (CollectionUtil.isEmpty(dailyPlanList)) {
            List<BaseCrossPlanPO> crossPlanPOList = baseCrossPlanMapper.selectByCrossId(crossId);
            if (CollectionUtil.isNotEmpty(crossPlanPOList)) {
                for (BaseCrossPlanPO baseCrossPlanPO : crossPlanPOList) {
                    RunningPlanDTO.DailyPlanListElement dailyPlan = new RunningPlanDTO.DailyPlanListElement();
                    dailyPlan.setPlanNo(baseCrossPlanPO.getPlanNo());
                    dailyPlan.setName(baseCrossPlanPO.getName());
                    dailyPlan.setTimeList(new ArrayList<>());
                    dailyPlanList.add(dailyPlan);
                }
            }
        } else {
            for (RunningPlanDTO.DailyPlanListElement dailyPlan : dailyPlanList) {
                List<RunningPlanDTO.TimeListElement> timeList = dailyPlan.getTimeList();
                for (RunningPlanDTO.TimeListElement timeListElement : timeList) {
                    String[] startEndTime = new String[2];
                    String startTime = timeListElement.getStartTime();
                    String endTime = timeListElement.getEndTime();
                    startEndTime[0] = startTime;
                    startEndTime[1] = endTime;
                    timeListElement.setStartEndTime(startEndTime);
                }
            }
        }
        return dailyPlanList;
    }

    private List<RunningPlanDTO.SchedulesPlanListElement> buildSchedulesPlanList(String crossId) {
        List<RunningPlanDTO.SchedulesPlanListElement> schedulesPlanList = new ArrayList<>();
        List<BaseCrossSchedulesPO> baseCrossSchedulesPOList = baseCrossSchedulesMapper.selectByCrossId(crossId);
        for (BaseCrossSchedulesPO baseCrossSchedulesPO : baseCrossSchedulesPOList) {
            String scheduleNo = baseCrossSchedulesPO.getScheduleNo();
            String name = baseCrossSchedulesPO.getName();
            Integer crossSchedulesPOId = baseCrossSchedulesPO.getId();
            RunningPlanDTO.SchedulesPlanListElement schedulesPlan = new RunningPlanDTO.SchedulesPlanListElement();
            schedulesPlan.setScheduleNo(scheduleNo);
            schedulesPlan.setName(name);
            schedulesPlan.setId(crossSchedulesPOId);
            schedulesPlan.setStatus(baseCrossSchedulesPO.getStatus());
            // 构造execList
            List<RunningPlanDTO.ExecListElement> execList = buildExecList(crossId, scheduleNo);
            schedulesPlan.setExecList(execList);
            schedulesPlanList.add(schedulesPlan);
        }
        return schedulesPlanList;
    }

    private List<RunningPlanDTO.ExecListElement> buildExecList(String crossId, String scheduleNo) {
        List<RunningPlanDTO.ExecListElement> execList = new ArrayList<>();
        // 获取调度ID
        BaseCrossSchedulesPO schedulesPO = baseCrossSchedulesMapper.selectByCrossIdAndScheduleNo(crossId, scheduleNo);
        Integer schedulesId = schedulesPO.getId();
        // 根据路口ID、调度ID和计划ID查询调度计划关系
        List<BaseCrossSchedulesPlanPO> baseCrossSchedulesPlanPOList =
                baseCrossSchedulesPlanMapper.selectByCrossIdAndSchedulesId(crossId, schedulesId);
        Map<Integer, List<BaseCrossSchedulesPlanPO>> collectMap = baseCrossSchedulesPlanPOList.stream()
                .collect(Collectors.groupingBy(BaseCrossSchedulesPlanPO::getPlanId));
        for (Map.Entry<Integer, List<BaseCrossSchedulesPlanPO>> entry : collectMap.entrySet()) {
            RunningPlanDTO.ExecListElement execListElement = new RunningPlanDTO.ExecListElement();
            List<Integer> week = new ArrayList<>();
            List<String> specialDateList = new ArrayList<>();
            Integer planId = entry.getKey();
            List<BaseCrossSchedulesPlanPO> secondList = entry.getValue();
            for (BaseCrossSchedulesPlanPO baseCrossSchedulesPlanPO : secondList) {
                Integer day = baseCrossSchedulesPlanPO.getWeek();
                if (day == 0) { // 特殊日期
                    Date specialDate = baseCrossSchedulesPlanPO.getSpecialDate();
                    SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
                    specialDateList.add(sdf.format(specialDate));
                } else {
                    week.add(day);
                }
            }
            execListElement.setWeek(week);
            execListElement.setSpecialDateList(specialDateList);
            BaseCrossPlanPO baseCrossPlanPO = baseCrossPlanMapper.selectById(planId);
            execListElement.setPlanName(baseCrossPlanPO.getName());
            execListElement.setPlanId(baseCrossPlanPO.getId());
            execList.add(execListElement);
        }
        return execList;
    }

    boolean isTimeIntersecting(String startTime1, String endTime1, String startTime2, String endTime2) {
        return !(endTime1.compareTo(startTime2) <= 0 || startTime1.compareTo(endTime2) >= 0);
    }


    private List<CrossDataHistPO> buildSpilloverEvents(List<CrossDataHistPO> crossDataHistPOList) {
        return buildEvents(crossDataHistPOList, CrossStatusEnum.SPILLOVER.getCode());
    }

    private List<CrossDataHistPO> buildUnbalanceEvents(List<CrossDataHistPO> crossDataHistPOList) {
        return buildEvents(crossDataHistPOList, CrossStatusEnum.UNBALANCE.getCode());
    }

    private List<CrossDataHistPO> buildCongestionEvents(List<CrossDataHistPO> crossDataHistPOList) {
        return buildEvents(crossDataHistPOList, CrossStatusEnum.CONGESTION.getCode());
    }

    private List<CrossDataHistPO> buildEvents(List<CrossDataHistPO> crossDataHistPOList, int status) {

        List<CrossDataHistPO> collect = crossDataHistPOList.stream()
                .filter(po -> po.getStatus() == status)
                .collect(Collectors.groupingBy(
                        CrossDataHistPO::getStartTime,
                        Collectors.collectingAndThen(
                                Collectors.maxBy(Comparator.comparingInt(CrossDataHistPO::getDuration)),
                                Optional::get
                        )
                ))
                .values()
                .stream()
                .collect(Collectors.toList());

        return collect;
    }

}
