package net.wanji.opt.service.signalcontrol.impl;

import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import net.wanji.common.framework.exception.FeignServiceException;
import net.wanji.common.framework.rest.JsonViewObject;
import net.wanji.databus.entity.develop.servicedevelop.develop.StatusCodeEnum;
import net.wanji.databus.vo.SchemeSendVO;
import net.wanji.feign.service.UtcFeignClients;
import net.wanji.opt.cache.CrossDirTurnPhaseCache;
import net.wanji.opt.dao.mapper.CrossSchemeOptLogMapper;
import net.wanji.opt.dto.CrossPhaseDTO;
import net.wanji.opt.po.base.CrossSchemeOptLogPO;
import net.wanji.opt.service.impl.CrossOptimizeServiceImpl;
import net.wanji.opt.service.signalcontrol.FeignProxyService;
import net.wanji.opt.vo.SchemeOptSendVO;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;

import java.time.LocalDateTime;
import java.time.ZoneOffset;
import java.util.*;

/**
 * @author duanruiming
 * @date 2023/03/02 9:11
 */
@Service
@Slf4j
@RequiredArgsConstructor
public class FeignProxyServiceImpl implements FeignProxyService {

    private final UtcFeignClients utcFeignClients;
    private final CrossSchemeOptLogMapper crossSchemeOptLogMapper;
    private final CrossDirTurnPhaseCache crossDirTurnPhaseCache;
    private final CrossOptimizeServiceImpl crossOptimizeService;

    @Override
    public JsonViewObject schemeOptSend(SchemeOptSendVO schemeOptSendVO) throws Exception {

        SchemeSendVO schemeSendVO = new SchemeSendVO();
        BeanUtils.copyProperties(schemeOptSendVO, schemeSendVO);
        JsonViewObject jsonViewObject = utcFeignClients.tempScheme(schemeOptSendVO);
        if (Objects.isNull(jsonViewObject) || jsonViewObject.getCode() != 200) {
            log.error("诊断优化手动下发优化方案，UTC服务调用异常", jsonViewObject);
            throw new FeignServiceException("诊断优化手动下发优化方案，UTC服务调用异常");
        }

        // 将优化记录插入方案优化记录表
        List<CrossSchemeOptLogPO> crossSchemeOptLogPOS = getCrossSchemeOptLogPOList(schemeOptSendVO);
        crossSchemeOptLogMapper.insertBatch(crossSchemeOptLogPOS);
        return jsonViewObject.success("手动下发优化方案成功");
    }

    @Override
    public JsonViewObject schemeOptRestore(String crossId) throws Exception {
        // 下发原始方案
        Map<String, CrossPhaseDTO> phaseMap = crossDirTurnPhaseCache.getPhaseDirTurnMap();
        SchemeSendVO schemeSendVO = crossOptimizeService.getSchemeSendVO(crossId, phaseMap, Collections.EMPTY_MAP);
        JsonViewObject jsonViewObject = utcFeignClients.schemeSend(schemeSendVO);
        if (Objects.isNull(jsonViewObject) || jsonViewObject.getCode() != 200) {
            log.error("诊断优化手动恢复优化方案，UTC服务调用异常！", jsonViewObject);
            throw new FeignServiceException("手动恢复优化方案，UTC服务调用异常");
        }
        return jsonViewObject.success("手动恢复优化方案成功");
    }

    /**
     * 获取方案优化数据，插入到数据库
     *
     * @param schemeOptSendVO
     * @return
     */
    private List<CrossSchemeOptLogPO> getCrossSchemeOptLogPOList(SchemeOptSendVO schemeOptSendVO) {
        List<CrossSchemeOptLogPO> crossSchemeOptLogPOS = new ArrayList<>();
        Map<String, CrossPhaseDTO> phaseDirTurnMap = crossDirTurnPhaseCache.getPhaseDirTurnMap();

        long dataBatchTimeLong = LocalDateTime.now().toEpochSecond(ZoneOffset.of("+8"));
        Map<String, Integer> phaseOffsetTimeMap = schemeOptSendVO.getPhaseOffsetTimeMap();
        String crossId = schemeOptSendVO.getCrossCode();
        List<SchemeSendVO.Pattern> patternList = schemeOptSendVO.getPatternList();
        for (SchemeSendVO.Pattern pattern : patternList) {
            String patternNo = pattern.getPatternNo();
            String offset = pattern.getOffset();

            List<SchemeSendVO.Pattern.Ring> rings = pattern.getRings();
            for (SchemeSendVO.Pattern.Ring ring : rings) {
                String ringNo = ring.getRingNo();

                List<SchemeSendVO.Pattern.Ring.Phase> phaseList = ring.getPhaseList();
                for (SchemeSendVO.Pattern.Ring.Phase phase : phaseList) {
                    String phaseNo = phase.getPhaseNo();
                    String greenTime = phase.getGreenTime();
                    Integer greenTimeOffset = phaseOffsetTimeMap.get(phaseNo);

                    for (Map.Entry<String, CrossPhaseDTO> entry : phaseDirTurnMap.entrySet()) {
                        CrossPhaseDTO crossPhaseDTO = entry.getValue();
                        if (StringUtils.equals(crossId, crossPhaseDTO.getCrossId()) && StringUtils.equals(phaseNo, crossPhaseDTO.getPhaseNo())) {
                            Integer dirType = crossPhaseDTO.getDirType();
                            String turnType = crossPhaseDTO.getTurnType();

                            CrossSchemeOptLogPO crossSchemeOptLogPO = new CrossSchemeOptLogPO();
                            crossSchemeOptLogPO.setCrossId(crossId);
                            crossSchemeOptLogPO.setSchemeId(Integer.valueOf(patternNo));
                            crossSchemeOptLogPO.setDirType(dirType);
                            crossSchemeOptLogPO.setTurnType(turnType);
                            crossSchemeOptLogPO.setOffset(Integer.valueOf(offset));
                            crossSchemeOptLogPO.setRingNo(Integer.valueOf(ringNo));
                            crossSchemeOptLogPO.setPhaseNo(phaseNo);
                            crossSchemeOptLogPO.setPhaseOrderId(Integer.valueOf(phase.getSort()));
                            crossSchemeOptLogPO.setOriGreenTime(Integer.valueOf(greenTime) - greenTimeOffset);
                            crossSchemeOptLogPO.setOptGreenTime(Integer.valueOf(greenTime));
                            crossSchemeOptLogPO.setOptTime(greenTimeOffset);
                            crossSchemeOptLogPO.setOptType(schemeOptSendVO.getOptType());
                            crossSchemeOptLogPO.setOptReason(schemeOptSendVO.getOptReason());
                            crossSchemeOptLogPO.setStartTime(new Date());
                            crossSchemeOptLogPO.setRelationFlag(schemeOptSendVO.getRelationFlag());
                            crossSchemeOptLogPO.setRelationCrossId(schemeOptSendVO.getRelationCrossId());
                            crossSchemeOptLogPO.setOptResult("1");
                            crossSchemeOptLogPO.setOptResultDesc(StatusCodeEnum.STATUS_00200.getDetail());
                            crossSchemeOptLogPO.setDataBatchTime((int) dataBatchTimeLong);
                        }
                    }
                }
            }
        }
        return crossSchemeOptLogPOS;
    }
}
